//******************************************************************************
//*
//*     FULLNAME:  Single-Chip Microcontroller Real-Time Operating System
//*
//*     NICKNAME:  scmRTOS
//*
//*     PROCESSOR: ARM Cortex-M0, Nordic Semiconductor nRF51822
//*
//*     TOOLKIT:   EWARM (IAR Systems)
//*
//*     PURPOSE:   Port Test File
//*
//*     Version: 5.1.0
//*
//*
//*     Copyright (c) 2003-2015, scmRTOS Team
//*
//*     Permission is hereby granted, free of charge, to any person 
//*     obtaining  a copy of this software and associated documentation 
//*     files (the "Software"), to deal in the Software without restriction, 
//*     including without limitation the rights to use, copy, modify, merge, 
//*     publish, distribute, sublicense, and/or sell copies of the Software, 
//*     and to permit persons to whom the Software is furnished to do so, 
//*     subject to the following conditions:
//*
//*     The above copyright notice and this permission notice shall be included 
//*     in all copies or substantial portions of the Software.
//*
//*     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
//*     EXPRESS  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
//*     MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
//*     IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
//*     CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
//*     TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
//*     THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//*
//*     =================================================================
//*     Project sources: https://github.com/scmrtos/scmrtos
//*     Documentation:   https://github.com/scmrtos/scmrtos/wiki/Documentation
//*     Wiki:            https://github.com/scmrtos/scmrtos/wiki
//*     Sample projects: https://github.com/scmrtos/scmrtos-sample-projects
//*     =================================================================
//*
//******************************************************************************
//* Cortex-M0 nRF51x IAR port, Akimov Vladimir, 'sevstels', Copyright (c) 2016

#include <scmRTOS.h>
#include "nrf_delay.h"
#include "nrf_gpio.h"
#include "ble400.h"

//------------------------------------------------------------------------------
//
//      Process types
//
typedef OS::process<OS::pr0, 400> TProc1;
typedef OS::process<OS::pr1, 400> TProc2;
typedef OS::process<OS::pr2, 400> TProc3;

//------------------------------------------------------------------------------
tick_count_t T;  // global variable for OS::GetTickCount testing

//------------------------------------------------------------------------------
//
//   Message "body"
//
class TSlon
{
public:
    TSlon() { }
    // feed the slon. For non-russians: slon == elephant ;)
    virtual void eat() = 0;      
};
//------------------------------------------------------------------------------
class TAfricanSlon : public TSlon
{
public:
    virtual void eat()
    {
        TCritSect cs; 

        ; // Make some action
    }
};
//------------------------------------------------------------------------------
class TIndianSlon : public TSlon
{
public:
    virtual void eat()
    {
        TCritSect cs; 

        ; // Make some action
    }
};

//------------------------------------------------------------------------------
TAfricanSlon African;   
TIndianSlon  Indian;   

// OS::channel object for 8 'TSlon*' items    
OS::channel<TSlon*, 8> SlonQueue; 

OS::TEventFlag Timer_Flag;

//For controller gpio test
const uint8_t leds_list[LEDS_NUMBER] = LEDS_LIST;

//------------------------------------------------------------------------------
void main()
{
  // Configure LED-pins as outputs.
  LEDS_CONFIGURE(LEDS_MASK);
  
  //Board LEDs tets
  for (int i = 0; i < LEDS_NUMBER; i++)
    {
       LEDS_INVERT(1 << leds_list[i]);
       nrf_delay_ms(500);
    }
   
  OS::run();
}

//------------------------------------------------------------------------------
template<> OS_PROCESS void TProc1::exec()
{
  int i = 0;

  for(;;)
    {
      //Timer_Flag.wait();
      ///SlonQueue.push(&African);
      
      LEDS_INVERT(1 << leds_list[i]);
      i++;
      if(i>LEDS_NUMBER) i=0;
      sleep(2);
      
    }     
}

//------------------------------------------------------------------------------
template<> OS_PROCESS void TProc2::exec()
{
  sleep();  
  for(;;)
    {
        T += OS::get_tick_count();
             
        sleep(1);
        SlonQueue.push(&Indian);
    }
}

//------------------------------------------------------------------------------
template<> OS_PROCESS void TProc3::exec()
{
  sleep(); 
  for(;;)
    {
        //--------------------------------------------------
        //
        //            Channel test
        //     Get data through channel
        //
        TSlon *p;
        SlonQueue.pop(p);     // get pointer from queue
        p->eat();             // feed the appropriate Slon
    }
}

//------------------------------------------------------------------------------
void OS::system_timer_user_hook()
{
    ///Timer_Flag.signal_isr();
}

//------------------------------------------------------------------------------
void OS::idle_process_user_hook()
{
  
}

//------------------------------------------------------------------------------
//
//      Process objects
//
TProc1 Proc1;
TProc2 Proc2;
TProc3 Proc3;

