//******************************************************************************
//*
//*     FULLNAME:  Single-Chip Microcontroller Real-Time Operating System
//*
//*     NICKNAME:  scmRTOS
//*
//*     PROCESSOR: ARM Cortex-M0, Nordic Semiconductor nRF51822
//*
//*     TOOLKIT:   EWARM (IAR Systems)
//*
//*     PURPOSE:  Custom system timer module
//*
//*     Version: v5.1.0
//*
//*
//*     Copyright (c) 2003-2016, scmRTOS Team
//*
//*     Permission is hereby granted, free of charge, to any person
//*     obtaining  a copy of this software and associated documentation
//*     files (the "Software"), to deal in the Software without restriction,
//*     including without limitation the rights to use, copy, modify, merge,
//*     publish, distribute, sublicense, and/or sell copies of the Software,
//*     and to permit persons to whom the Software is furnished to do so,
//*     subject to the following conditions:
//*
//*     The above copyright notice and this permission notice shall be included
//*     in all copies or substantial portions of the Software.
//*
//*     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//*     EXPRESS  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
//*     MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//*     IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
//*     CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
//*     TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH
//*     THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//*
//*     =================================================================
//*     Project sources: https://github.com/scmrtos/scmrtos
//*     Documentation:   https://github.com/scmrtos/scmrtos/wiki/Documentation
//*     Wiki:            https://github.com/scmrtos/scmrtos/wiki
//*     Sample projects: https://github.com/scmrtos/scmrtos-sample-projects
//*     =================================================================
//*
//******************************************************************************
//* Cortex-M0 nRF51x IAR port, Akimov Vladimir, 'sevstels', Copyright (c) 2016

//#include "nrf51.h"
//#include "nrf_timer.h"
#include "nrf.h"
#include <scmRTOS.h>

//------------------------------------------------------------------------------
// Custom system timer initialization
// Sets SysTick timer interrupt priority a bit higher than lowest one.
// Configures SysTick timer to interrupt with frequency SYSTICKINTRATE.
//------------------------------------------------------------------------------
//#pragma optimize=none
#pragma weak os_init_custom_system_timer
extern "C" void os_init_custom_system_timer()
{
  //Timer0 busy for Nordic BLE stack, use Timer1 like SysTick timer
  
  // Stop timer
  NRF_TIMER1->TASKS_STOP = 1;
    
  //Select Timer mode
  NRF_TIMER1->MODE = TIMER_MODE_MODE_Timer;
    
  //Set prescaler 1000 Hz 1us resolution
  NRF_TIMER1->PRESCALER = 4;
  
  //Width = 16 bit
  NRF_TIMER1->BITMODE = TIMER_BITMODE_BITMODE_16Bit;
  
  // Clear timer
  NRF_TIMER1->TASKS_CLEAR = 1;
      
  //Compare (period) value
  NRF_TIMER1->CC[0] = 32000;
  
  //Interrupt enable
  NRF_TIMER1->INTENSET = (TIMER_INTENSET_COMPARE0_Enabled << 
                          TIMER_INTENSET_COMPARE0_Pos);

  NRF_TIMER1->SHORTS = (TIMER_SHORTS_COMPARE0_CLEAR_Enabled << 
                        TIMER_SHORTS_COMPARE0_CLEAR_Pos);

  //interrupt handler allow
  NVIC_SetPriority(TIMER1_IRQn, 255);
  NVIC_ClearPendingIRQ(TIMER1_IRQn);
  NVIC_EnableIRQ(TIMER1_IRQn);  
  
  //Run timer
  NRF_TIMER1->TASKS_START = 1;  
}

//------------------------------------------------------------------------------
// Custom system timer stop
//------------------------------------------------------------------------------
INLINE void LOCK_SYSTEM_TIMER(void) { NRF_TIMER1->TASKS_STOP = 1; }
//------------------------------------------------------------------------------
// Custom system timer run
//------------------------------------------------------------------------------
INLINE void UNLOCK_SYSTEM_TIMER(void){ NRF_TIMER1->TASKS_START = 1;} 

//------------------------------------------------------------------------------
// Timer1 Interrupt handler
//------------------------------------------------------------------------------
extern "C" OS_INTERRUPT void TIMER1_IRQHandler(void)
{ 
  OS::scmRTOS_ISRW_TYPE TISRW;
    
  if(NRF_TIMER1->EVENTS_COMPARE[0])
   {

     NRF_TIMER1->EVENTS_COMPARE[0] = 0;

     #if scmRTOS_SYSTIMER_HOOK_ENABLE == 1
     OS::system_timer_user_hook();
     #endif

     OS::Kernel.system_timer();

     #if scmRTOS_SYSTIMER_NEST_INTS_ENABLE == 0
     DISABLE_NESTED_INTERRUPTS();
     #endif
     
     NRF_TIMER1->SHORTS = TIMER_SHORTS_COMPARE0_CLEAR_Msk;
   }
}
