//******************************************************************************
//*
//*     FULLNAME:  Single-Chip Microcontroller Real-Time Operating System
//*
//*     NICKNAME:  scmRTOS
//*
//*     PROCESSOR: ARM Cortex-M0, Nordic Semiconductor nRF51822
//*
//*     TOOLKIT:   EWARM (IAR Systems)
//*
//*     PURPOSE:   Target Dependent Stuff Header. Declarations And Definitions
//*
//*     Version: 5.00
//*
//*     $Revision: 1 $
//*     $Date:: 2016-12-15 #$
//*
//*     Copyright (c) 2003-2012, Harry E. Zhurov
//*
//*     Permission is hereby granted, free of charge, to any person
//*     obtaining  a copy of this software and associated documentation
//*     files (the "Software"), to deal in the Software without restriction,
//*     including without limitation the rights to use, copy, modify, merge,
//*     publish, distribute, sublicense, and/or sell copies of the Software,
//*     and to permit persons to whom the Software is furnished to do so,
//*     subject to the following conditions:
//*
//*     The above copyright notice and this permission notice shall be included
//*     in all copies or substantial portions of the Software.
//*
//*     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//*     EXPRESS  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
//*     MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//*     IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
//*     CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
//*     TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH
//*     THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//*
//*     =================================================================
//*     See http://scmrtos.sourceforge.net for documentation, latest
//*     information, license and contact details.
//*     =================================================================
//*
//******************************************************************************
//* Cortex-M3/M4(F) GCC port by Anton B. Gusev aka AHTOXA, Copyright (c) 2012
//* Cortex-M0 GCC port by Sergey A. Borshch, Copyright (c) 2011

#ifndef scmRTOS_CORTEX_M0_IAR_H
#define scmRTOS_CORTEX_M0_IAR_H

//------------------------------------------------------------------------------
// Compiler and Target checks
// For M0 core : __ARM6__ IAR define

#ifndef __IAR_SYSTEMS_ICC__
#error "This file should be compiled with IAR C++ ARM Compiler"
#endif // __IAR_SYSTEMS_ICC__

#if (!defined __ARM6__)
#error "This file must be compiled for ARMv6 (Cortex-M0) processors only."
#endif

#if (!defined NRF51)
#error "Define NRF51 for Nordic Semiconductor, nrF51x BLE family."
#endif

extern void LOCK_SYSTEM_TIMER();
extern void UNLOCK_SYSTEM_TIMER();

//------------------------------------------------------------------------------
// Compiler specific attributes
//
#ifndef INLINE
#define INLINE   _Pragma("inline=forced") inline
#endif

#ifndef NORETURN
#define NORETURN __noreturn
#endif

//------------------------------------------------------------------------------
// Target specific types
//
typedef uint32_t stack_item_t;
typedef uint32_t status_reg_t;

//------------------------------------------------------------------------------
// Configuration macros
//
#define OS_PROCESS              __task
#define OS_INTERRUPT            extern "C"
#define DUMMY_INSTR()           __no_operation()
#define INLINE_PROCESS_CTOR     INLINE

//------------------------------------------------------------------------------
// Separate return stack not required
#define SEPARATE_RETURN_STACK   0

//------------------------------------------------------------------------------
// Software interrupt stack switching not supported in Cortex-M port
// because processor implements hardware stack switching.
// So, system timer ISR wrapper can't be chosen at project level
//
#define scmRTOS_ISRW_TYPE       TISRW

//------------------------------------------------------------------------------
// scmRTOS Context Switch Scheme
//
// The macro defines a context switch manner. Value 0 sets direct context
// switch in the scheduler and in the OS ISRs. This is the primary method.
// Value 1 sets the second way to switch context - by using of software
// interrupt. See documentation for details.
// Cortex-M port supports software interrupt switch method only.
//
#define  scmRTOS_CONTEXT_SWITCH_SCHEME  1

//------------------------------------------------------------------------------
// Include project-level configurations
// !!! The order of includes is important !!!
//
#include "scmRTOS_CONFIG.h"
#include "scmRTOS_TARGET_CFG.h"
#include <scmRTOS_defs.h>

//------------------------------------------------------------------------------
// Target-specific configuration macros
//
#ifdef  scmRTOS_USER_DEFINED_STACK_PATTERN
#define scmRTOS_STACK_PATTERN scmRTOS_USER_DEFINED_STACK_PATTERN
#else
#define scmRTOS_STACK_PATTERN   0x67894523UL
#endif

//------------------------------------------------------------------------------
// Interrupt and Interrupt Service Routines support
//
#include <intrinsics.h>
#define enable_interrupts()  __enable_interrupt()
#define disable_interrupts() __disable_interrupt()
#define get_interrupt_state  __get_interrupt_state
#define set_interrupt_state  __set_interrupt_state

#if scmRTOS_OBSOLETE_NAMES == 1

INLINE status_reg_t GetInterruptState( )       { return get_interrupt_state();}
INLINE void SetInterruptState(status_reg_t sr) { set_interrupt_state(sr);}

INLINE void EnableInterrupts()  { enable_interrupts();  }
INLINE void DisableInterrupts() { disable_interrupts(); }

#endif // scmRTOS_OBSOLETE_NAMES

//------------------------------------------------------------------------------
// The Critical Section Wrapper
//
#if scmRTOS_USER_DEFINED_CRITSECT_ENABLE == 0
class TCritSect
{
public:
  TCritSect () : StatusReg(get_interrupt_state()) { disable_interrupts(); }
 ~TCritSect() { set_interrupt_state(StatusReg); }

private:
  status_reg_t StatusReg;
};
#endif // scmRTOS_USER_DEFINED_CRITSECT_ENABLE

//   Uncomment macro value below for system_timer() and
//   context_switch_hook() run in critical section.
// 
//   This is useful (and necessary) when target processor has hardware 
//   enabled nested interrupts.
//   User can define own macros using user-defined TCritSect capabilities.
//
//   Cortex-M have nested interrupts but interrupts are disabled
//   during context switch ISR. So, critical section is needed
//   for system timer routine and not needed for context switcher.
// 
#define SYS_TIMER_CRIT_SECT() TCritSect cs
#define CONTEXT_SWITCH_HOOK_CRIT_SECT()

//------------------------------------------------------------------------------
// __builtin_clz
//
INLINE static uint32_t  popcnt( uint32_t x )
{
    x -= ((x >> 1) & 0x55555555);
    x = (((x >> 2) & 0x33333333) + (x & 0x33333333));
    x = (((x >> 4) + x) & 0x0f0f0f0f);
    x += (x >> 8);
    x += (x >> 16);
    return x & 0x0000003f;
}

INLINE static uint32_t  __builtin_clz( uint32_t x )
{
    x |= (x >> 1);
    x |= (x >> 2);
    x |= (x >> 4);
    x |= (x >> 8);
    x |= (x >> 16);
    return 32 - popcnt(x);
}

//------------------------------------------------------------------------------
// Priority stuff
//
namespace OS
{
INLINE OS::TProcessMap get_prio_tag(const uint_fast8_t pr) 
        { return static_cast<OS::TProcessMap> (1 << pr); }

#if scmRTOS_PRIORITY_ORDER == 0
    INLINE uint_fast8_t highest_priority(TProcessMap pm)
    {
        extern TPriority const PriorityTable[];

        #if scmRTOS_PROCESS_COUNT < 6
            return PriorityTable[pm];
        #else
            uint32_t x = pm;
            x = x & -x;        // Isolate rightmost 1-bit.

                               // x = x * 0x450FBAF
            x = (x << 4) | x;  // x = x*17.
            x = (x << 6) | x;  // x = x*65.
            x = (x << 16) - x; // x = x*65535.

            return PriorityTable[x >> 26];
        #endif  // scmRTOS_PROCESS_COUNT < 6
    }
#else
    INLINE uint_fast8_t highest_priority(TProcessMap pm)
    {
    	return 31 - __builtin_clz(pm);
    }
#endif // scmRTOS_PRIORITY_ORDER
}

namespace OS
{
  INLINE void enable_context_switch()  { enable_interrupts(); }
  INLINE void disable_context_switch() { disable_interrupts(); }
}

#include <OS_Kernel.h>

namespace OS
{
    //--------------------------------------------------------------------------
    //   NAME       :   OS ISR support
    //
    //  PURPOSE    :   Implements common actions on interrupt enter and exit
    //                 under the OS
    class TISRW
    {
    public:
        INLINE  TISRW()  { ISR_Enter(); }
        INLINE  ~TISRW() { ISR_Exit();  }

    private:
        //-----------------------------------------------------
        INLINE void ISR_Enter()
        {
            TCritSect cs;
            Kernel.ISR_NestCount++;
        }
        //-----------------------------------------------------
        INLINE void ISR_Exit()
        {
            TCritSect cs;
            if(--Kernel.ISR_NestCount) return;
            Kernel.sched_isr();
        }
        //-----------------------------------------------------
    };

    // No software interrupt stack switching provided,
    // TISRW_SS declared to be the same as TISRW for porting compatibility
    #define TISRW_SS  TISRW

} // namespace OS
//------------------------------------------------------------------------------

#endif // scmRTOS_CORTEX_M0_H
//------------------------------------------------------------------------------

